/***********************************
 * #include <disclaimer.h>  //this C code is going make this program run faster :)
 * Author: Narendranath D.
 ***********************************/

import com.ibm.mq.*;
import java.util.Hashtable;
import java.io.*;
import java.util.*;

public class MQHelper {
    private Hashtable properties;
    private String errString = "";


    public MQHelper(Hashtable options){
	properties = new Hashtable(options);
    }

    public String getErrorString(){
	return errString;
    }

    public String getMessage(String qm, String qn){
        String message = "";
        try {	

                MQQueueManager qMgr = new MQQueueManager(qm, properties);
                int openOptions = MQC.MQOO_FAIL_IF_QUIESCING + MQC.MQOO_INPUT_SHARED;

                //Now specify the queue that we wish to open, and the open options...
                MQQueue queue = qMgr.accessQueue(qn,openOptions);

                MQMessage retrievedMessage =new MQMessage();
                //Set the get message options...
                MQGetMessageOptions gmo =new MQGetMessageOptions();//accept the defaults

                //get the message off the queue...
                queue.get(retrievedMessage,gmo);
                retrievedMessage.characterSet = ((Integer) properties.get("mqcharset")).intValue();

                int length = retrievedMessage.getMessageLength();
                //System.out.print("length is: " + length);
                message = retrievedMessage.readString(length /((Integer)properties.get("mqcharlength")).intValue());
                //Close the queue...
                queue.close();
                //Disconnect from the queue manager
                qMgr.disconnect();
                
        }
        //If an error has occurred in the above,try to identify what went wrong
        //Was it an MQSeries error?
        catch (MQException ex)
        {
            if (ex.completionCode == 2 && ex.reasonCode == MQException.MQRC_NO_MSG_AVAILABLE)
            {
                message = "no messages to get.";
            } else {
                message = "An MQSeries error occurred :Completion code " +
                ex.completionCode + "Reason code " + ex.reasonCode;
            }
        }
        //Was it a Java buffer space error?
        catch (java.io.IOException ex)
        {
                message = "An error occurred whilst writing to the message buffer:"+ ex;
        }
        return message;
    }
    
    
    public String browseMessage(String qm, String qn){
        String msg = "";
        try {	
                MQQueueManager qMgr = new MQQueueManager(qm, properties);
                int openOptions = MQC.MQOO_FAIL_IF_QUIESCING + MQC.MQOO_BROWSE;

                //Now specify the queue that we wish to open, and the open options...
                MQQueue queue = qMgr.accessQueue(qn,openOptions);

                MQMessage retrievedMessage =new MQMessage();

                //Set the get message options...
                MQGetMessageOptions gmo =new MQGetMessageOptions();//accept the defaults
                gmo.options = MQC.MQGMO_NO_WAIT + MQC.MQGMO_FAIL_IF_QUIESCING + MQC.MQGMO_BROWSE_FIRST;

                //browse the first message on the queue...
                queue.get(retrievedMessage,gmo);
                retrievedMessage.characterSet = ((Integer) properties.get("mqcharset")).intValue();

                int length = retrievedMessage.getMessageLength();
                //System.out.print("length is: " + length);

                msg = retrievedMessage.readString(length /((Integer)properties.get("mqcharlength")).intValue());

                //Close the queue...
                queue.close();
                //Disconnect from the queue manager
                qMgr.disconnect();
        }

        //If an error has occurred in the above,try to identify what went wrong
        //Was it an MQSeries error?
        catch (MQException ex)
        {
            if (ex.completionCode == 2 && ex.reasonCode == MQException.MQRC_NO_MSG_AVAILABLE)
            {
                msg = "no more messages to browse.";
            } else {
                msg = "An MQSeries error occurred :Completion code " +
                ex.completionCode + "Reason code " + ex.reasonCode;
            }
 
        }
        //Was it a Java buffer space error?
        catch (java.io.IOException ex)
        {
                msg = "An error occurred while writing to the message buffer:"+ ex;
        }
        return msg;
    }
    
    public boolean postMessage(String qm, String qn, String msg, String replyq){
        try {	
            MQQueueManager qMgr = new MQQueueManager(qm, properties);
            int openOptions =MQC.MQOO_FAIL_IF_QUIESCING + MQC.MQOO_OUTPUT ;

            //Now specify the queue that we wish to open, and the open options...
            MQQueue queue = qMgr.accessQueue(qn,openOptions);
            MQMessage mqmsg =new MQMessage();
            mqmsg.characterSet = ((Integer) properties.get("mqcharset")).intValue();
            mqmsg.writeString(msg);
            mqmsg.replyToQueueName = replyq;

            //Set the put message options...
            MQPutMessageOptions pmo =new MQPutMessageOptions();

            //put the message on the queue...
            queue.put(mqmsg,pmo);

            //Close the queue...
            queue.close();
            //Disconnect from the queue manager
            qMgr.disconnect();
            return true;
        }
        //If an error has occurred in the above,try to identify what went wrong
        //Was it an MQSeries error?
        catch (MQException ex)
        {
            errString ="An MQSeries error occurred :Completion code " +
            ex.completionCode + "Reason code " + ex.reasonCode;
        }
        //Was it a Java buffer space error?
        catch (java.io.IOException ex)
        {
            errString = "An error occurred whilst writing to the message buffer:"+ ex;
        }

        return false;
    }

    public boolean getMessages(String qm, String qn, String dirName, String baseName){
        ArrayList msgList = new ArrayList();
        boolean status = true;
        try {	
	    MQQueueManager qMgr = new MQQueueManager(qm, properties);
	    int openOptions = MQC.MQOO_INQUIRE + MQC.MQOO_FAIL_IF_QUIESCING + MQC.MQOO_INPUT_SHARED;
            MQQueue queue = qMgr.accessQueue(qn,openOptions);
            int depth = queue.getCurrentDepth();
            if (depth == 0)
            {
                 errString = "no messages to browse...";
                 return false;
            }

            //Set the get message options...
            MQGetMessageOptions gmo =new MQGetMessageOptions();
            gmo.options = MQC.MQGMO_NO_WAIT + MQC.MQGMO_FAIL_IF_QUIESCING;

            String message = "";
            while(true)
            {                        
                MQMessage retrievedMessage =new MQMessage();
                try
                {
                    queue.get(retrievedMessage,gmo);
                    retrievedMessage.characterSet = ((Integer) properties.get("mqcharset")).intValue();
                    int length = retrievedMessage.getMessageLength();
                    message = retrievedMessage.readString(length /((Integer)properties.get("mqcharlength")).intValue());
                    msgList.add(message);
                    retrievedMessage.clearMessage();
                }
                catch (IOException ex)
                {
                    errString = "IOException during GET: " + ex.getMessage();
                    status = false;
                    break;
                }
                catch (MQException ex)
                {
                    if (!(ex.completionCode == 2 && ex.reasonCode == MQException.MQRC_NO_MSG_AVAILABLE))
                    {
                        errString = "GET Exception: " + ex;
                        status = false;
                    }
                    break;
                }
            }//end of while       
            queue.close();
            qMgr.disconnect();
        }
   	//If an error has occurred in the above,try to identify what went wrong
   	catch (MQException ex)
   	{
	    errString ="An MQSeries error occurred :Completion code " +
	    ex.completionCode + " Reason code " + ex.reasonCode;
            status = false;
   	}
                
        //Write the the browsed messages in the ArrayList to the files
        FileHelper fh = new FileHelper();
        try
        {
            fh.writeFiles(msgList, dirName, baseName);
        }
        catch (java.io.IOException ex)
        {
            errString = "IO Error occured while creating the message files: " + ex;
        }
        return status;
        
    }
 
    
    public boolean browseMessages(String qm, String qn, String dirName, String baseName) {
        ArrayList msgList = new ArrayList();
        boolean status = true;
        try {	
	    MQQueueManager qMgr = new MQQueueManager(qm, properties);
	    int openOptions = MQC.MQOO_INQUIRE + MQC.MQOO_BROWSE + MQC.MQOO_FAIL_IF_QUIESCING + MQC.MQOO_INPUT_SHARED;
            MQQueue queue = qMgr.accessQueue(qn,openOptions);
            int depth = queue.getCurrentDepth();
            if (depth == 0)
            {
                 errString = "no messages to browse...";
                 return false;
            }

            //Set the get message options...
            MQGetMessageOptions gmo =new MQGetMessageOptions();
            gmo.options = MQC.MQGMO_BROWSE_NEXT + MQC.MQGMO_NO_WAIT + MQC.MQGMO_FAIL_IF_QUIESCING;

            String message = "";
            while(true)
            {                        
                MQMessage retrievedMessage =new MQMessage();
                try
                {
                    queue.get(retrievedMessage,gmo);
                    retrievedMessage.characterSet = ((Integer) properties.get("mqcharset")).intValue();
                    int length = retrievedMessage.getMessageLength();
                    message = retrievedMessage.readString(length /((Integer)properties.get("mqcharlength")).intValue());
                    msgList.add(message);
                    retrievedMessage.clearMessage();
                }
                catch (IOException ex)
                {
                    errString = "IOException during GET: " + ex.getMessage();
                    status = false;
                    break;
                }
                catch (MQException ex)
                {
                    if (!(ex.completionCode == 2 && ex.reasonCode == MQException.MQRC_NO_MSG_AVAILABLE))
                    {
                        errString = "GET Exception: " + ex;
                        status = false;
                    }
                    break;
                }
            }//end of while       
            queue.close();
            qMgr.disconnect();
        }
   	//If an error has occurred in the above,try to identify what went wrong
   	catch (MQException ex)
   	{
	    errString ="An MQSeries error occurred :Completion code " +
	    ex.completionCode + " Reason code " + ex.reasonCode;
            status = false;
   	}
                
        //Write the the browsed messages in the ArrayList to the files
        FileHelper fh = new FileHelper();
        try
        {
            fh.writeFiles(msgList, dirName, baseName);
        }
        catch (java.io.IOException ex)
        {
            errString = "IO Error occured while creating the message files: " + ex;
        }
        return status;
    }
 
    
    
    
    
    
 
    public boolean clearMessages(String qm, String qn){
        boolean status = true;
        try {	
	    MQQueueManager qMgr = new MQQueueManager(qm, properties);
	    int openOptions = MQC.MQOO_INQUIRE + MQC.MQOO_FAIL_IF_QUIESCING + MQC.MQOO_INPUT_SHARED;
            MQQueue queue = qMgr.accessQueue(qn,openOptions);
            int depth = queue.getCurrentDepth();
            if (depth == 0)
            {
                 errString = "no messages to clear...";
                 return false;
            }

            //Set the get message options...
            MQGetMessageOptions gmo =new MQGetMessageOptions();
            gmo.options = MQC.MQGMO_NO_WAIT + MQC.MQGMO_FAIL_IF_QUIESCING + MQC.MQGMO_ACCEPT_TRUNCATED_MSG;

            String message = "";
            while(true)
            {                        
                MQMessage retrievedMessage =new MQMessage();
                try
                {
                    queue.get(retrievedMessage,gmo);
                    //retrievedMessage.characterSet = ((Integer) properties.get("mqcharset")).intValue();
                    //int length = retrievedMessage.getMessageLength();
                    //message = retrievedMessage.readString(length /((Integer)properties.get("mqcharlength")).intValue());
                    //msgList.add(message);
                    retrievedMessage.clearMessage();
                }
                catch (IOException ex)
                {
                    errString = "IOException during Clear Messages: " + ex.getMessage();
                    status = false;
                    break;
                }
                catch (MQException ex)
                {
                    if (!(ex.completionCode == 2 && ex.reasonCode == MQException.MQRC_NO_MSG_AVAILABLE))
                    {
                        errString = "GET Exception: " + ex;
                        status = false;
                    }
                    break;
                }
            }//end of while       
            queue.close();
            qMgr.disconnect();
        }
   	//If an error has occurred in the above,try to identify what went wrong
   	catch (MQException ex)
   	{
	    errString ="An MQSeries error occurred :Completion code " +
	    ex.completionCode + " Reason code " + ex.reasonCode;
            status = false;
   	}
                
        return status;
    }
    
    
    
    
    
    
    public boolean postMessages(String qm, String qn, String msgtopost, String replyq, int n){
        try {	
            MQQueueManager qMgr = new MQQueueManager(qm, properties);
            int openOptions =MQC.MQOO_FAIL_IF_QUIESCING + MQC.MQOO_OUTPUT ;

            //Now specify the queue that we wish to open, and the open options...
            MQQueue queue = qMgr.accessQueue(qn,openOptions);
            MQMessage mqmsg =new MQMessage();
            mqmsg.characterSet = ((Integer) properties.get("mqcharset")).intValue();
            mqmsg.writeString(msgtopost);
            mqmsg.replyToQueueName = replyq;

            //Set the get message options...
            MQPutMessageOptions pmo =new MQPutMessageOptions();

            for (int i=0; i < n; i++) {
                queue.put(mqmsg, pmo);
                mqmsg.messageId = MQC.MQMI_NONE;
                //mqmsg.clearMessage();
                //mqmsg.writeString(msgtopost);
            }
            errString = "";
            //Close the queue...
            queue.close();
            //Disconnect from the queue manager
            qMgr.disconnect();
            return true;
        }

        //If an error has occurred in the above,try to identify what went wrong
        //Was it an MQSeries error?
        catch (MQException ex)
        {
                errString ="An MQSeries error occurred :Completion code " +
                ex.completionCode + "Reason code " + ex.reasonCode;
                return false;
        }
        //Was it a Java buffer space error?
        catch (java.io.IOException ex)
        {
                errString = "An error occurred whilst writing to the message buffer:"+ ex;
                return false;
        }
    
    }
}